import java.io.*;

public class InputLoop implements Observable<String> {

    private List<Observer<String>> observers;

    private InputLoop () {
        observers = List.empty();
    }

    public static InputLoop create() {
        return new InputLoop();
    }

    public void registerObserver (Observer<String> ob) {
        observers = List.cons(ob,observers);   // mutation
    }
    
    public void notifyObservers (String data) {
        FuncIterator<Observer<String>> it = observers.getFuncIterator();
        while (it.hasElement()) {
            it.current().notify(data);
            it = it.advance();
        }
    }

    private String getInput () {

        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
        String response = "";
    
        try {
            response = br.readLine();
            if (response==null) {
                return "";
            }
        } catch (IOException ioe) {
            System.out.println("IO error reading from terminal\n");
            System.exit(1);
        }
        return response;
    }


    public void loop () {

        String response;
        while (true) {
            System.out.print("> ");
            response = getInput();
            notifyObservers(response);
        }
    }


}